/*
 * This file contains the code used by various IRQ handling routines:
 * asking for different IRQ's should be done through these routines
 * instead of just grabbing them. Thus setups with different IRQ numbers
 * shouldn't result in any weird surprises, and installing new handlers
 * should be easier.
 *
 * The Z80 is rather simpler than most. We have one IRQ source and we run
 * through 'logical IRQ's' in order.
 */

#include <linuxmt/config.h>
#include <linuxmt/types.h>
#include <linuxmt/errno.h>
#include <linuxmt/sched.h>
#include <linuxmt/errno.h>
#include <linuxmt/timex.h>

#include <arch/system.h>
#include <arch/irq.h>
#include <arch/io.h>
#include <arch/keyboard.h>

struct irqaction 
{
	void (*handler)();
	unsigned long flags;
	unsigned long mask;
	char *name;
};

#define IRQF_BAD	1		/* This IRQ is bad if it occurs */
	
static struct irqaction irq_action[16]= { NULL,};

static int irq_off=0;		/* We use a software disable */

/*
 *	Called by the assembler hooks (RST)
 */
 
void do_IRQ( regs)
void *regs;
{
	int i;
	struct irqaction *irq=irq_action;

	if(irq_off)
		return;
			
	for(i=0;i16;i++)
	{
		if(irq->handler)
			irq->handler(i,regs);
		irq++;
	}
}

/*
 *	Low level IRQ control.
 */
 
unsigned long __save_flags()
{
	return irq_off;
}

void restore_flags(flags)
unsigned long flags;
{
	irq_off=flags;
}

void cli()
{
	irq_off=1;
}

void sti()
{
	irq_off=0;
}
		
int request_irq(irq, handler, irqflags, devname)
int irq;
void (*handler);
unsigned long irqflags;
char *devname;	
{
	struct irqaction *action;
	unsigned long flags;
	
	if (irq > 15)
		return -EINVAL;
	action = irq + irq_action;
	if(action->handler)
		return -EBUSY;
	if(!handler)
		return -EINVAL;
	save_flags(flags);
	cli();
	action->handler = handler;
	action->flags = irqflags;
	action->mask = 0;
	action->name = devname;
	restore_flags(flags);
	return 0;
}

void free_irq(irq)
unsigned int irq;
{
	struct irqaction * action = irq + irq_action;
	unsigned long flags;

	if (irq > 15) {
		printk("Trying to free IRQ%d\n",irq);
		return;
	}
	if (!action->handler) {
		printk("Trying to free free IRQ%d\n",irq);
		return;
	}
	save_flags(flags);
	cli();
	action->handler = NULL;
	action->flags = 0;
	action->mask = 0;
	action->name = NULL;
	restore_flags(flags);
}

/*
 *	Test timer tick routine
 */

unsigned long jiffies=0;

void timer_tick()
{
	jiffies++;
}

/*
 *	IRQ setup.
 *
 */
 
void init_IRQ()
{
	struct irqaction *irq=irq_action + 16;
	int ct;
	
	irqtab_init();			/* Store DS */
	
	/*
	 *	Set off the initial timer interrupt handler
	 */
	 
	if(request_irq(0, timer_tick, 0, "timer"))
		panic("Unable to get timer");

	/*
	 *	Set off the initial keyboard interrupt handler
	 */
	 
	if(request_irq(1, keyboard_irq, 0, "keyboard"))
		panic("Unable to get keyboard");
	 	
}

